/*
	File: RemoteTypes.r
	Contains: rez template for 'IRCD' resource
	
	Copyright  1993 by Apple Computer, Inc.
	All rights reserved.
*/


/*
	IRCD resource is a way of representing IR remote control codes

	Xmit	+---------------+	 +-+  +---+ +----+
							|	 | |  |   | |	 |
	Idle					+----+ +--+   +-+	 +--------------+-------+

			|--- lead in ---|					 |--- repeat ---|
												 |------ lead out ------|
*/


//----------------------------------------------------------------------
//		TMPL - ResEdit template
//----------------------------------------------------------------------
type 'TMPL' {
	array item {
		pstring;			// field name
		literal longint;	// field type
	};
};

resource 'TMPL' (128, "IRCD") {
	{
		"name",		'TNAM',
		"timeBase",	'DLNG',
		"leadIn",	'DLNG',
		"repeat",	'DLNG',
		"leadOut",	'DLNG',
		"pad",		'DWRD',
		"count",	'OCNT',
		"*****",	'LSTC',
		"time",		'DLNG',
		"*****",	'LSTE'
	}
};


//----------------------------------------------------------------------
//		IRCD - IR Command Code
//----------------------------------------------------------------------
type 'IRCD' {
	literal longint;						// name
	unsigned longint;						// timeBase, in miscroseconds
	unsigned longint;						// leadIn duration, in timeBase units
	unsigned longint;						// repeat duration, in timeBase units
	unsigned longint;						// leadOut duration, in timeBase units
	
	unsigned longint = $$CountOf(BitTimes);
	array BitTimes {
		unsigned longint;					// bit duration in timeBase units
	};
};


