/*
	File: Sony.r
	Contains: Sony command codes for Newton Remote Control
	
	Copyright  1993 by Apple Computer, Inc.  All rights reerved.
	Written by $mike cremer with bits from Don Louv
	
	Rez -c RSED -t rsrc Sony.r -o Sony.rsrc
*/

/*
	Sony encodes each command as an AGC burst (2400 s)
	followed by a 7 bit button field and a 5 bit device
	field, transmitted lsb to msb.
	
	A "M" bit is the mark condition
	A "0" bit is encoded as 600 s idle followed by a 600 s pulse.
	A "1" bit is encoded as 600 s idle followed by a 1200 s pulse.
	
	Example:
	Using device 17 (CD player) and button 50 (CDPlay).
	The CD "play" command is therefore:
	
	M M M M   1 0 0 0 1   0 1 1 0 0 1 0
	  AGC		 CD			 CDPlay
	
	but the device/button field is sent lsb to msb, so
	really the bit stream looks like:
	
	M M M M   0 1 0 0 1 1 0   1 0 0 0 1
	
	The waveform for the "play" command looks like this:

      . M  . 0 . 1  . 0 . 0 . 1  . 1  . 0 . 1  . 0 . 0 . 0 . 1  .
      +----+ +-+ +--+ +-+ +-+ +--+ +--+ +-+ +--+ +-+ +-+ +-+ +--+
	  |    | | | |  | | | | | |  | |  | | | |  | | | | | | | |  |
	--+    +-+ +-+  +-+ +-+ +-+  +-+  +-+ +-+  +-+ +-+ +-+ +-+  +--
	    4   1 1 1 2  1 1 1 1 1 2  1 2  1 1 1 2  1 1 1 1 1 1 1 2

	The top numbers are the bit format of the command; '.' marks
	each bit boundary.  The bottom numbers are bit times which will
	produce the encoded waveform.
*/

#include "RemoteTypes.r"

resource 'IRCD' (200, "SonyCDPlay") {
	'cdpl',
	600,		// timeBase, in microseconds
	4,			// leadIn, in timeBase units
	74,			// repeat, in timeBase units (~44 ms)
	833,		// leadOut, in timeBase units (~500 ms)
	{
	1, 1, 1, 2,
	1, 1, 1, 1,
	1, 2, 1, 2,
	1, 1, 1, 2,
	1, 1, 1, 1,
	1, 1, 1, 2
	}
};

resource 'IRCD' (201, "SonyCDRew") {
	'cdrv',
	600,		// timeBase, in microseconds
	4,			// leadIn, in timeBase units
	74,			// repeat, in timeBase units (~44 ms)
	833,		// leadOut, in timeBase units (~500 ms)
	{
	1, 2, 1, 2,
	1, 1, 1, 1,
	1, 2, 1, 2,
	1, 1, 1, 2,
	1, 1, 1, 1,
	1, 1, 1, 2
	}
};

resource 'IRCD' (202, "SonyCDFF") {
	'cdfw',
	600,		// timeBase, in microseconds
	4,			// leadIn, in timeBase units
	74,			// repeat, in timeBase units (~44 ms)
	833,		// leadOut, in timeBase units (~500 ms)
	{
	1,1,1,1,
	1,2,1,1,
	1,2,1,2,
	1,1,1,2,
	1,1,1,1,
	1,1,1,2
	}
};

resource 'IRCD' (203, "SonyCDTrackDown") {
	'cdtd',
	600,		// timeBase, in microseconds
	4,			// leadIn, in timeBase units
	74,			// repeat, in timeBase units (~44 ms)
	833,		// leadOut, in timeBase units (~500 ms)
	{
	1,1,1,1,
	1,1,1,1,
	1,2,1,2,
	1,1,1,2,
	1,1,1,1,
	1,1,1,2
	}
};

resource 'IRCD' (204, "SonyCDTrackUp") {
	'cdtu',
	600,		// timeBase, in microseconds
	4,			// leadIn, in timeBase units
	74,			// repeat, in timeBase units (~44 ms)
	833,		// leadOut, in timeBase units (~500 ms)
	{
	1,2,1,1,
	1,1,1,1,
	1,2,1,2,
	1,1,1,2,
	1,1,1,1,
	1,1,1,2
	}
};

resource 'IRCD' (205, "SonyCDStop") {
	'cdsp',
	600,		// timeBase, in microseconds
	4,			// leadIn, in timeBase units
	74,			// repeat, in timeBase units (~44 ms)
	833,		// leadOut, in timeBase units (~500 ms)
	{
	1,1,1,1,
	1,1,1,2,
	1,2,1,2,
	1,1,1,2,
	1,1,1,1,
	1,1,1,2
	}
};

resource 'IRCD' (206, "SonyCDPause") {
	'cdpa',
	600,		// timeBase, in microseconds
	4,			// leadIn, in timeBase units
	74,			// repeat, in timeBase units (~44 ms)
	833,		// leadOut, in timeBase units (~500 ms)
	{
	1,2,1,1,
	1,1,1,2,
	1,2,1,2,
	1,1,1,2,
	1,1,1,1,
	1,1,1,2
	}
};

resource 'IRCD' (207, "SonyCDScanDown") {
	'cdsd',
	600,		// timeBase, in microseconds
	4,			// leadIn, in timeBase units
	74,			// repeat, in timeBase units (~44 ms)
	833,		// leadOut, in timeBase units (~500 ms)
	{
	1,1,1,2,
	1,1,1,2,
	1,2,1,2,
	1,1,1,2,
	1,1,1,1,
	1,1,1,2
	}
};


resource 'IRCD' (208, "SonyCDScanUp") {
	'cdsu',
	600,		// timeBase, in microseconds
	4,			// leadIn, in timeBase units
	74,			// repeat, in timeBase units (~44 ms)
	833,		// leadOut, in timeBase units (~500 ms)
	{
	1,2,1,2,
	1,1,1,2,
	1,2,1,2,
	1,1,1,2,
	1,1,1,1,
	1,1,1,2
	}
};




resource 'IRCD' (209, "CH1") {
	'CH01',
	600,		// timeBase, in microseconds
	4,			// leadIn, in timeBase units
	74,			// repeat, in timeBase units (~44 ms)
	833,		// leadOut, in timeBase units (~500 ms)
	{
	1,1,1,1,
	1,1,1,1,
	1,1,1,1,
	1,1,1,2,
	1,1,1,1,
	1,1,1,1,
	}
};


resource 'IRCD' (210, "CH2") {
	'CH02',
	600,		// timeBase, in microseconds
	4,			// leadIn, in timeBase units
	74,			// repeat, in timeBase units (~44 ms)
	833,		// leadOut, in timeBase units (~500 ms)
	{
	1,2,1,1,
	1,1,1,1,
	1,1,1,1,
	1,1,1,2,
	1,1,1,1,
	1,1,1,1,
	}
};


resource 'IRCD' (211, "CH3") {
	'CH03',
	600,		// timeBase, in microseconds
	4,			// leadIn, in timeBase units
	74,			// repeat, in timeBase units (~44 ms)
	833,		// leadOut, in timeBase units (~500 ms)
	{
	1,1,1,2,
	1,1,1,1,
	1,1,1,1,
	1,1,1,2,
	1,1,1,1,
	1,1,1,1,
	}
};


resource 'IRCD' (212, "CH4") {
	'CH04',
	600,		// timeBase, in microseconds
	4,			// leadIn, in timeBase units
	74,			// repeat, in timeBase units (~44 ms)
	833,		// leadOut, in timeBase units (~500 ms)
	{
	1,2,1,2,
	1,1,1,1,
	1,1,1,1,
	1,1,1,2,
	1,1,1,1,
	1,1,1,1,
	}
};


resource 'IRCD' (213, "CH4") {
	'CH04',
	600,		// timeBase, in microseconds
	4,			// leadIn, in timeBase units
	74,			// repeat, in timeBase units (~44 ms)
	833,		// leadOut, in timeBase units (~500 ms)
	{
	1,1,1,1,
	1,2,1,1,
	1,1,1,1,
	1,1,1,2,
	1,1,1,1,
	1,1,1,1,
	}
};


resource 'IRCD' (214, "CH5") {
	'CH05',
	600,		// timeBase, in microseconds
	4,			// leadIn, in timeBase units
	74,			// repeat, in timeBase units (~44 ms)
	833,		// leadOut, in timeBase units (~500 ms)
	{
	1,2,1,1,
	1,2,1,1,
	1,1,1,1,
	1,1,1,2,
	1,1,1,1,
	1,1,1,1,
	}
};


resource 'IRCD' (215, "CH6") {
	'CH06',
	600,		// timeBase, in microseconds
	4,			// leadIn, in timeBase units
	74,			// repeat, in timeBase units (~44 ms)
	833,		// leadOut, in timeBase units (~500 ms)
	{
	1,1,1,2,
	1,2,1,1,
	1,1,1,1,
	1,1,1,2,
	1,1,1,1,
	1,1,1,1,
	}
};


resource 'IRCD' (216, "CH7") {
	'CH07',
	600,		// timeBase, in microseconds
	4,			// leadIn, in timeBase units
	74,			// repeat, in timeBase units (~44 ms)
	833,		// leadOut, in timeBase units (~500 ms)
	{
	1,2,1,2,
	1,2,1,1,
	1,1,1,1,
	1,1,1,2,
	1,1,1,1,
	1,1,1,1,
	}
};


resource 'IRCD' (217, "CH8") {
	'CH08',
	600,		// timeBase, in microseconds
	4,			// leadIn, in timeBase units
	74,			// repeat, in timeBase units (~44 ms)
	833,		// leadOut, in timeBase units (~500 ms)
	{
	1,1,1,1,
	1,1,1,2,
	1,1,1,1,
	1,1,1,2,
	1,1,1,1,
	1,1,1,1,
	}
};


resource 'IRCD' (218, "CH9") {
	'CH09',
	600,		// timeBase, in microseconds
	4,			// leadIn, in timeBase units
	74,			// repeat, in timeBase units (~44 ms)
	833,		// leadOut, in timeBase units (~500 ms)
	{
	1,2,1,1,
	1,1,1,2,
	1,1,1,1,
	1,1,1,2,
	1,1,1,1,
	1,1,1,1,
	}
};


resource 'IRCD' (219, "CH10") {
	'CH10',
	600,		// timeBase, in microseconds
	4,			// leadIn, in timeBase units
	74,			// repeat, in timeBase units (~44 ms)
	833,		// leadOut, in timeBase units (~500 ms)
	{
	1,1,1,2,
	1,1,1,2,
	1,1,1,1,
	1,1,1,2,
	1,1,1,1,
	1,1,1,1,
	}
};


resource 'IRCD' (220, "Enter") {
	'Entr',
	600,		// timeBase, in microseconds
	4,			// leadIn, in timeBase units
	74,			// repeat, in timeBase units (~44 ms)
	833,		// leadOut, in timeBase units (~500 ms)
	{
	1,2,1,2,
	1,1,1,2,
	1,1,1,1,
	1,1,1,2,
	1,1,1,1,
	1,1,1,1,
	}
};


resource 'IRCD' (221, "VolUp") {
	'VolU',
	600,		// timeBase, in microseconds
	4,			// leadIn, in timeBase units
	74,			// repeat, in timeBase units (~44 ms)
	833,		// leadOut, in timeBase units (~500 ms)
	{
	1,1,1,2,
	1,1,1,1,
	1,2,1,1,
	1,1,1,2,
	1,1,1,1,
	1,1,1,1,
	}
};


resource 'IRCD' (222, "VolDown") {
	'VolD',
	600,		// timeBase, in microseconds
	4,			// leadIn, in timeBase units
	74,			// repeat, in timeBase units (~44 ms)
	833,		// leadOut, in timeBase units (~500 ms)
	{
	1,2,1,2,
	1,1,1,1,
	1,2,1,1,
	1,1,1,2,
	1,1,1,1,
	1,1,1,1,
	}
};


resource 'IRCD' (223, "Power") {
	'Powr',
	600,		// timeBase, in microseconds
	4,			// leadIn, in timeBase units
	74,			// repeat, in timeBase units (~44 ms)
	833,		// leadOut, in timeBase units (~500 ms)
	{
	1,2,1,1,
	1,2,1,1,
	1,2,1,1,
	1,1,1,2,
	1,1,1,1,
	1,1,1,1,
	}
};


resource 'IRCD' (224, "CHUp") {
	'CHUp',
	600,		// timeBase, in microseconds
	4,			// leadIn, in timeBase units
	74,			// repeat, in timeBase units (~44 ms)
	833,		// leadOut, in timeBase units (~500 ms)
	{
	1,1,1,1,
	1,1,1,1,
	1,2,1,1,
	1,1,1,2,
	1,1,1,1,
	1,1,1,1,
	}
};


resource 'IRCD' (225, "CHDown") {
	'CHDn',
	600,		// timeBase, in microseconds
	4,			// leadIn, in timeBase units
	74,			// repeat, in timeBase units (~44 ms)
	833,		// leadOut, in timeBase units (~500 ms)
	{
	1,2,1,1,
	1,1,1,1,
	1,2,1,1,
	1,1,1,2,
	1,1,1,1,
	1,1,1,1,
	}
};


resource 'IRCD' (226, "AudioMode") {
	'AudM',
	600,		// timeBase, in microseconds
	4,			// leadIn, in timeBase units
	74,			// repeat, in timeBase units (~44 ms)
	833,		// leadOut, in timeBase units (~500 ms)
	{
	1,2,1,2,
	1,2,1,1,
	1,2,1,1,
	1,1,1,2,
	1,1,1,1,
	1,1,1,1,
	}
};


resource 'IRCD' (227, "TVVTR") {
	'TVVT',
	600,		// timeBase, in microseconds
	4,			// leadIn, in timeBase units
	74,			// repeat, in timeBase units (~44 ms)
	833,		// leadOut, in timeBase units (~500 ms)
	{
	1,2,1,1,
	1,2,1,1,
	1,1,1,2,
	1,1,1,2,
	1,1,1,1,
	1,1,1,1,
	}
};


resource 'IRCD' (228, "Mute") {
	'Mute',
	600,		// timeBase, in microseconds
	4,			// leadIn, in timeBase units
	74,			// repeat, in timeBase units (~44 ms)
	833,		// leadOut, in timeBase units (~500 ms)
	{
	1,1,1,1,
	1,2,1,1,
	1,2,1,1,
	1,1,1,2,
	1,1,1,1,
	1,1,1,1,
	}
};


resource 'IRCD' (229, "Display") {
	'Disp',
	600,		// timeBase, in microseconds
	4,			// leadIn, in timeBase units
	74,			// repeat, in timeBase units (~44 ms)
	833,		// leadOut, in timeBase units (~500 ms)
	{
	1,1,1,2,
	1,1,1,2,
	1,2,1,2,
	1,1,1,2,
	1,1,1,1,
	1,1,1,1,
	}
};


resource 'IRCD' (230, "Sleep") {
	'Slep',
	600,		// timeBase, in microseconds
	4,			// leadIn, in timeBase units
	74,			// repeat, in timeBase units (~44 ms)
	833,		// leadOut, in timeBase units (~500 ms)
	{
	1,1,1,2,
	1,2,1,1,
	1,2,1,2,
	1,1,1,2,
	1,1,1,1,
	1,1,1,1,
	}
};